local json = require("json")
local utils = require("utils")
local const = require("const")

local driver_utils = {}

-- set single driver configuration to driver configuration
-- @driver_name: string, driver name
-- @config_new: table, configuration of single driver
-- function driver_utils.set_driver_conf(driver_name, config_new)
--     local ok, config
--     local config_raw = utils.get_driver_config(driver_name)
--     if (config_raw) then
--         ok, msg = pcall(json.decode, config_raw)
--         if (not ok) then 
--             utils.log_error("get driver config failed" .. msg)
--             utils.error_exit(500, "set config failed")
--         end
--         config = msg
--     else   
--         config = {}
--     end
--     
--     config["DriverConfig"] = config_new
--     utils.set_driver_config(driver_name, json.encode(config))
-- 
-- end

-- function driver_utils.get_driver_conf(driver_name)
--     local config_raw = utils.get_driver_config(driver_name)
--     if (not config_raw) then
--         return {}
--     end
--     local ok, msg = pcall(json.decode, config_raw)
--     if (ok) then 
--         local config = msg
--         if (config["DriverConfig"]) then 
--             return config["DriverConfig"]
--         else
--             return {}
--         end
--     end
-- 
--     utils.log_error("get driver config failed" .. msg)
--     utils.error_exit(500, "get config failed")
-- end

function driver_utils.get_driver_info(driver_name)
    local driver_info = {}
    local driver_type = nil

    driver_list = driver_utils.get_driver_list( )
    for _, drv in ipairs(driver_list) do
        if (driver_name == drv[1]) then
            driver_type = drv[2]
            break
        end
    end

    if (nil == driver_type) then
        utils.log_warn("invalid driver name" .. driver_name)
        utils.error_exit(400, "ginvalid driver name")
    end

    driver_info["DriverName"] = driver_name
    driver_info["DriverType"] = driver_type
    driver_info["DriverConfig"] = driver_utils.get_driver_conf(driver_name)
    return driver_info
end


local function get_dev_num( driver_name )
    -- {"code":0,"message":"success","params":{"devNum":1,"devList":["a1NkFqUzGCG_AirConditioningIvan"]}}
    ok, msg_raw, err = utils.dbus_call_edge(const.dimu.wkn,
                        const.dimu.obj,
                        const.dimu.inf,
                        "getLocalDeviceList",
                        "driverName=" .. driver_name)
    if (ok and msg_raw) then
        ok, msg = pcall(json.decode, msg_raw[1])
        if (ok and 0 == msg["code"]) then
            return msg["params"]["devNum"]
        end
    end

    return 0
end


-- get driver list from dbus daemon by org.freedesktop.DBus.ListNames
--function driver_utils.get_driver_list( )
--    -- get driver list from dbus daemon
--
--    --[[ sample of return message
--        array [
--      org.freedesktop.DBus   iot.driver.facepp   iot.dmp.dimu  iot.driver.gateway_monitor  iot.driver.photoCollect  ]
--    ]]--
--    local ok, msg, err = utils.dbus_call_edge(
--                            "org.freedesktop.DBus",
--                            "/org/freedesktop/DBus",
--                            "org.freedesktop.DBus",
--                            "ListNames")
--    local empty = true
--    local driver_list = {}
--    if (ok and msg) then
--        name_list = string.gsub(msg, "^%s+array %[\n(.+)%s*%]", "%1")
--        for item in string.gmatch(name_list, '[^%s]+') do
--            name, n = string.gsub(item, "iot.driver.(%w+)", "%1");
--            if (1 == n) then
--                local item = {}
--                table.insert(item, name)
--                table.insert(item, "All")
--                table.insert(item, tostring(os.time()))
--                table.insert(item, get_dev_num(name))
--                table.insert(driver_list, item)
--                empty = false
--            end
--        end
--    end
--
--    if (empty) then
--        return nil
--    else
--        return driver_list
--    end
--end


function driver_utils.get_driver_list( )
    local driver_list = {}

    -- {"code":0,"message":"success","params":[["LedDriver",2,"1527859921103"],["gateway_monitor",2,"1527859037366"]]}
    local ok, raw_msg, err = utils.dbus_call_edge(
                                const.dimu.wkn,
                                const.dimu.obj,
                                const.dimu.inf,
                                "getDriverList",
                                "")
    if (ok and raw_msg) then
        local ok, msg = pcall(json.decode, raw_msg[1]) 
        if (ok and 0 == msg["code"]) then
            for _, drv in ipairs(msg["params"]) do
                local item = {}
                table.insert(item, drv[1])
                table.insert(item, "All")
                table.insert(item, tostring(math.floor(tonumber(drv[3])/1000)))
                table.insert(item, get_dev_num(drv[1]))
                table.insert(driver_list, item)
            end

            return driver_list
        end
    end
    utils.log_error("get driver list failed")
    utils.error_exit(500, "get driver list failed")
end


function driver_utils.get_driver_list_detail( )
    local driver_list_detail = {}
    local driver_list = driver_utils.get_driver_list( )
    if (nil == driver_list) then
        return nil
    end

    for n, driver_info in ipairs(driver_list) do
        local item = {}
        item["DriverName"] = driver_info[1]
        item["DriverType"] = driver_info[2]
        item["LoadTime"] = driver_info[3]
        item["DeviceNum"] = driver_info[4]
        item["DriverState"] = "working"
        table.insert(driver_list_detail, item)
    end

    return driver_list_detail
end

return driver_utils
