local json = require("json")
local utils = require("utils")
local utils_device = require("utils_device")
local pl_utils = require("pl.utils")
local request_method = ngx.var.request_method
local const = require("const")
local pl_path = require("pl.path")

utils.check_login()
utils.check_permission("DeviceManage")

local function get_standard_tsl(pk)
    _, msg = utils.get_data(const.data_key.tsl.standard_tsl_prefix..pk)

    if (msg == nil) then
        msg = "{}"
    end

    return msg
end

local function get_product_name_from_TSL(tsl)
    local args = json.decode(tsl)
    if args then
        if args["profile"] then
            return args["profile"]["productName"]
        end
    else
        return nil
    end
end

function split(str, sep)
    local t = {}
    string.gsub(str, '[^'..sep..']+', function(w) table.insert(t, w) end )
    return t
end

if ("POST" == request_method) then
    local utils_header = require("utils_header")
    local params = utils_header.check_public_boby_data()
    local pk = params["ProductKey"]
    local page_size = tonumber(params["PageSize"])
    local cur_page = tonumber(params["CurrentPage"])
    local sort_method = params["SortMethod"]

    if (sort_method == nil) or ((sort_method ~= "TimeDesc") and (sort_method ~= "TimeAsc")) then
        sort_method = "TimeDesc"
    end

    page_size = utils.check_page_size_allow_zero(page_size)

    utils.check_current_page(cur_page)

    local cur_dir = pl_path.currentdir()
    local path_tool_cfg = cur_dir.."/../tool_config"
    local path_lib = cur_dir.."/../../lib/"
    local cmd = "LD_LIBRARY_PATH="..path_lib.."  "..path_tool_cfg.." -d3"
    local ret_code, act_code, stdout, stderr = pl_utils.executeex(cmd, false)
    
    if not ret_code then
        utils.error_exit(400, "failed to call tool config")
    end
    local str_1 =  split(stdout, "\n")
    local msg_pk_dn = {}
    local tab_mapping_pk_product_name = {}

    local total_count = 0
    local tab_tmp = {}
    for k, v in pairs(str_1) do
        if k > 2 then
            local str_2 = split(v, "\t")
            if str_2[3] == "Unused" and (pk == nil or string.len(pk) == 0 or pk == str_2[1]) then
            total_count = total_count + 1
                local item = {}
                item["ProductKey"] = str_2[1]
                item["DeviceName"] = str_2[2]

            if tab_mapping_pk_product_name[ str_2[1] ] == nil then
            local tsl = get_standard_tsl(str_2[1])
            tab_mapping_pk_product_name[ str_2[1] ] = get_product_name_from_TSL( tsl )
            end
            item["ProductName"] = tab_mapping_pk_product_name[ str_2[1] ]

                table.insert(tab_tmp, item)

            end
        end
    end
    if page_size <= 0 then
        page_size = total_count
    end

    local min, max = utils.get_item_index(total_count, page_size, cur_page)
    for k, v in pairs(tab_tmp) do
        if (k >= min and k <= max) then
        table.insert(msg_pk_dn, v)
        end
    end

    res = {}
    res["TotalNum"] = total_count
    res["PageSize"] = page_size
    res["CurrentPage"] = cur_page
    res["List"] = msg_pk_dn
    utils.error_exit(200, json.encode(res))
end

utils.error_exit(400, "unsupported request method")
