#!/usr/bin/env python3
# -*- coding: utf-8 -*-


# ====#====#====#====
# author: $(USER)
# file: $(NAME)
# time: $(DATE)
# ====#====#====#====

import sys

sys.path.append("../../lib/")

from leda_python import leda
from leda_python.mbus import MbusBase
import hashlib
import logging
import threading
import signal
import json
import time
import os

_logger = logging.getLogger(__name__)

driverObj = leda.LedaModule()


def test_for_getDeviceList(driverObj):
    print('>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>')
    driver_name = "abc123"
    driverObj.moduleInit(driver_name)

    product_key = 'abcdefghigk'

    profile_dict = {
        "properties": [],
        "services": [],
        "events": []
    }

    profile = json.dumps(profile_dict)

    class ConfigCallback(leda.LedaConfigCallback):
        def deviceConfigCB(self, moduleName, moduleConfig):
            print("configCallback: moduleName %s, moduleConfigMsg...ok!!: %s", moduleName, moduleConfig)

    obj = ConfigCallback()

    driverObj.registerDeviceConfigCallback(obj)

    # Ok
    class Demo_device(leda.BaseDeviceCallback):
        def __init__(self):
            self.name = "deviceMbus"

        def callService(self, name, input):
            method_name = 'method_1'
            print("method%s, inparam: %s" % (name, input))

            if (name == method_name):
                outPut = {
                    "key1": "xxx",
                    "key2": "yyy",
                    "key3": "zzz",
                }
                return 0, outPut

        def getprofile_cb(self):
            profile_dict = {
                "properties": [],
                "services": [],
                "events": []
            }

            return json.dumps(profile_dict)

        def getProperties(self, input):
            _logger.debug("method: get property inparams:%s" % input)

            input_json = input
            output_json = {}

            for property in input_json:
                output_json[property] = "xxx"

            return 0, output_json

        def setProperties(self, input):
            _logger.debug("method: set property inparams:%s" % input)

            output = {}
            return 0, output

    # errCode
    class Demo_device2(leda.BaseDeviceCallback):
        def __init__(self):
            self.name = "deviceMbus"

        def callService(self, name, input):
            method_name = 'method_1'
            _logger.debug("method%s, inparam: %s" % (name, input))

            if (name == method_name):
                outPut = {
                    "key1": "xxx",
                    "key2": "yyy",
                    "key3": "zzz",
                }
                return -1, outPut

        def getProperties(self, input):
            _logger.debug("method: get property inparams:%s" % input)

            input_json = json.loads(input)
            output_json = {}

            for property in input_json:
                output_json[property] = "xxx"

            return -1, output_json

        def setProperties(self, input):
            _logger.debug("method: set property inparams:%s" % input)

            output = {}
            return -1, output

    # no callBack
    class Demo_device3(leda.BaseDeviceCallback):
        def __init__(self):
            self.name = "deviceMbus"

        def getprofile_cb(self):
            profile_dict = {
                "properties": [],
                "services": [],
                "events": []
            }

            return profile_dict

    # code type error
    class Demo_device4(leda.BaseDeviceCallback):
        def __init__(self):
            self.name = "deviceMbus"

        def callService(self, name, input):
            method_name = 'method_1'
            print("method%s, inparam: %s" % (name, input))

            if (name == method_name):
                outPut = {
                    "key1": "xxx",
                    "key2": "yyy",
                    "key3": "zzz",
                }
                return '123', json.dumps(outPut)

        def getProperties(self, input):
            _logger.debug("method: get property inparams:%s" % input)

            input_json = input
            output_json = {}

            for property in input_json:
                output_json[property] = "xxx"

            return '123', json.dumps(output_json)

        def setProperties(self, input):
            _logger.debug("method: set property inparams:%s" % input)

            output = {}
            return '123', json.dumps(output)

    # errCode
    class Demo_device5(leda.BaseDeviceCallback):
        def __init__(self):
            self.name = "deviceMbus"

        def callService(self, name, input):
            method_name = 'method_1'
            print("method%s, inparam: %s" % (name, input))

            if (name == method_name):
                outPut = {
                    "key1": "xxx",
                    "key2": "yyy",
                    "key3": "zzz",
                }
                return 12, outPut

        def getprofile_cb(self):
            profile_dict = {
                "properties": [],
                "services": [],
                "events": []
            }

            return json.dumps(profile_dict)

        def getProperties(self, input):
            _logger.debug("method: get property inparams:%s" % input)

            input_json = input
            output_json = {}

            for property in input_json:
                output_json[property] = "xxx"

            return 12, output_json

        def setProperties(self, input):
            _logger.debug("method: set property inparams:%s" % input)

            output = {}
            return 12, output

    tmpName = 'deviceaabb'
    deviceObj = []

    for i in range(0, 60):
        device_name = tmpName + str(i)

        if (i <= 9):  # success
            bus_callback_object = Demo_device()
            try:
                TmpdeviceObj = driverObj.deviceRegister(device_name, product_key, profile,
                                                        bus_callback_object)
            except:
                print("device register: Err")
        elif ((i >= 10) and (i <= 19)):  # errCode
            bus_callback_object = Demo_device2()
            try:
                TmpdeviceObj = driverObj.deviceRegister(device_name, product_key, profile,
                                                        bus_callback_object)
            except:
                print("device register: Err")

        elif ((i >= 20) and (i <= 29)):  # no register get method
            bus_callback_object = Demo_device3()
            try:
                TmpdeviceObj = driverObj.deviceRegister(device_name, product_key, profile,
                                                        bus_callback_object)
            except:
                print("device register: Err")

        elif ((i >= 30) and (i <= 39)):  # return invalid type code
            bus_callback_object = Demo_device4()
            try:
                TmpdeviceObj = driverObj.deviceRegister(device_name, product_key, profile,
                                                        bus_callback_object)
            except:
                print("device register: Err")

        elif ((i >= 40) and (i <= 49)):  # return invalid code
            bus_callback_object = Demo_device5()
            try:
                TmpdeviceObj = driverObj.deviceRegister(device_name, product_key, profile,
                                                        bus_callback_object)
            except:
                print("device register: Err")

        elif ((i >= 50) and (i <= 59)):
            bus_callback_object = Demo_device()  # test offline status
            try:
                TmpdeviceObj = driverObj.deviceRegister(device_name, product_key, profile,
                                                        bus_callback_object)
            except:
                print("device register: Err")

        deviceObj.append(TmpdeviceObj)

    for i in range(0, 50):
        deviceObj[i].online()

    for i in range(50, 60):
        deviceObj[i].offline()


g_Flag = False


def Signal_handle(signum, frame):
    global g_Flag
    print('recv a signal: %s' % (frame))
    # driverObj.driver_exit()
    g_Flag = True


if (__name__ == "__main__"):
    logging.basicConfig(level=logging.DEBUG,
                        format='%(asctime)s-%(threadName)-9s-%(filename)s[line:%(lineno)d]-%(levelname)s: %(message)s')
    _logger.debug("app testing...")
    test_for_getDeviceList(driverObj)
    signal.signal(signal.SIGINT, Signal_handle)
    while (1):
        time.sleep(0.5)
        if (g_Flag): break
